=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2011-02-19 15:26:45 +0000
+++ bzrlib/branch.py	2011-02-19 16:42:16 +0000
@@ -1592,10 +1592,7 @@
         try:
             transport = a_bzrdir.get_branch_transport(None, name=name)
             format_string = transport.get_bytes("format")
-            format = format_registry.get(format_string)
-            if isinstance(format, MetaDirBranchFormatFactory):
-                return format()
-            return format
+            return format_registry.get(format_string)
         except errors.NoSuchFile:
             raise errors.NotBranchError(path=transport.base, bzrdir=a_bzrdir)
         except KeyError:
@@ -1784,6 +1781,7 @@
         return False
 
     @classmethod
+    @deprecated_method(deprecated_in((2, 4, 0)))
     def unregister_format(klass, format):
         format_registry.remove(format)
 
@@ -1804,7 +1802,6 @@
     bzrlib.plugins.loom.formats).
     """
 
-    @deprecated_method(deprecated_in((2, 4, 0)))
     def __init__(self, format_string, module_name, member_name):
         """Create a MetaDirBranchFormatFactory.
 
@@ -2387,14 +2384,8 @@
 
     def register(self, format):
         """Register a new branch format."""
-        # Metadir formats have a network name of their format string, and get
-        # registered as factories.
-        if isinstance(format, MetaDirBranchFormatFactory):
-            super(BranchFormatRegistry, self).register(
-                format.get_format_string(), format.__class__)
-        else:
-            super(BranchFormatRegistry, self).register(
-                format.get_format_string(), format)
+        super(BranchFormatRegistry, self).register(
+            format.get_format_string(), format)
 
     def remove(self, format):
         """Remove a registered branch format."""
@@ -2407,21 +2398,32 @@
         This is mainly useful to allow custom branch formats, such as
         older Bazaar formats and foreign formats, to be tested
         """
-        self._extra_formats.append(format)
+        self._extra_formats.append(registry._ObjectGetter(format))
         network_format_registry.register(
             format.network_name(), format.__class__)
 
+    def register_extra_lazy(self, module_name, member_name):
+        """Register a branch format lazily.
+        """
+        self._extra_formats.append(
+            registry._LazyObjectGetter(module_name, member_name))
+
     @classmethod
     def unregister_extra(self, format):
-        self._extra_formats.remove(format)
+        self._extra_formats.remove(registry._ObjectGetter(format))
 
     def _get_all(self):
         result = []
         for name, fmt in self.iteritems():
-            if isinstance(fmt, MetaDirBranchFormatFactory):
-                fmt = fmt()
-            result.append(fmt)
-        return result + self._extra_formats
+            if callable(fmt):
+                fmt = fmt()
+            result.append(fmt)
+        for objgetter in self._extra_formats:
+            fmt = objgetter.get_obj()
+            if callable(fmt):
+                fmt = fmt()
+            result.append(fmt)
+        return result
 
     def set_default(self, format):
         self._default_format = format
@@ -3644,6 +3646,8 @@
         :param run_hooks: Private parameter - if false, this branch
             is being called because it's the master of the primary branch,
             so it should not run its hooks.
+            is being called because it's the master of the primary branch,
+            so it should not run its hooks.
         :param _override_hook_target: Private parameter - set the branch to be
             supplied as the target_branch to pull hooks.
         :param local: Only update the local branch, and not the bound branch.

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2011-02-19 15:29:28 +0000
+++ bzrlib/tests/test_branch.py	2011-02-19 16:42:16 +0000
@@ -54,7 +54,7 @@
     def test_get_set_default_format(self):
         # set the format and then set it back again
         old_format = _mod_branch.format_registry.get_default()
-        _mod_branch.BranchFormat.set_default_format(SampleBranchFormat())
+        _mod_branch.format_registry.set_default(SampleBranchFormat())
         try:
             # the default branch format is used by the meta dir format
             # which is not the default bzrdir format at this point
@@ -62,7 +62,7 @@
             result = dir.create_branch()
             self.assertEqual(result, 'A branch')
         finally:
-            _mod_branch.BranchFormat.set_default_format(old_format)
+            _mod_branch.format_registry.set_default(old_format)
         self.assertEqual(old_format,
                          _mod_branch.format_registry.get_default())
 
@@ -180,25 +180,14 @@
             self.failUnless(isinstance(found_format, format.__class__))
         check_format(_mod_branch.BzrBranchFormat5(), "bar")
 
-    def test_extra_format(self):
-        dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
-        SampleSupportedBranchFormat().initialize(dir)
-        format = SampleExtraBranchFormat()
-        _mod_branch.BranchFormat.register_extra_format(format)
-        self.addCleanup(_mod_branch.BranchFormat.unregister_extra_format,
-            format)
-        self.assertTrue(format in _mod_branch.BranchFormat.get_formats())
-        self.assertEquals(format,
-            _mod_branch.network_format_registry.get("extra"))
-
     def test_find_format_factory(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         SampleSupportedBranchFormat().initialize(dir)
         factory = _mod_branch.MetaDirBranchFormatFactory(
             SampleSupportedBranchFormatString,
             "bzrlib.tests.test_branch", "SampleSupportedBranchFormat")
-        _mod_branch.BranchFormat.register_format(factory)
-        self.addCleanup(_mod_branch.BranchFormat.unregister_format, factory)
+        _mod_branch.format_registry.register(factory)
+        self.addCleanup(_mod_branch.format_registry.remove, factory)
         b = _mod_branch.Branch.open(self.get_url())
         self.assertEqual(b, "opened supported branch.")
 
@@ -216,13 +205,15 @@
                           dir)
 
     def test_register_unregister_format(self):
+        # Test the deprecated format registration functions
         format = SampleBranchFormat()
         # make a control dir
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         # make a branch
         format.initialize(dir)
         # register a format for it.
-        _mod_branch.BranchFormat.register_format(format)
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+            _mod_branch.BranchFormat.register_format, format)
         # which branch.Open will refuse (not supported)
         self.assertRaises(errors.UnsupportedFormatError,
                           _mod_branch.Branch.open, self.get_url())
@@ -232,10 +223,53 @@
             format.open(dir),
             bzrdir.BzrDir.open(self.get_url()).open_branch(unsupported=True))
         # unregister the format
-        _mod_branch.BranchFormat.unregister_format(format)
+        self.applyDeprecated(symbol_versioning.deprecated_in((2, 4, 0)),
+            _mod_branch.BranchFormat.unregister_format, format)
         self.make_branch_and_tree('bar')
 
 
+class TestBranchFormatRegistry(tests.TestCase):
+
+    def setUp(self):
+        super(TestBranchFormatRegistry, self).setUp()
+        self.registry = _mod_branch.BranchFormatRegistry()
+
+    def test_default(self):
+        self.assertIs(None, self.registry.get_default())
+        format = SampleBranchFormat()
+        self.registry.set_default(format)
+        self.assertEquals(format, self.registry.get_default())
+
+    def test_register_unregister_format(self):
+        format = SampleBranchFormat()
+        self.registry.register(format)
+        self.assertEquals(format,
+            self.registry.get("Sample branch format."))
+        self.registry.remove(format)
+        self.assertRaises(KeyError, self.registry.get,
+            "Sample branch format.")
+
+    def test_get_all(self):
+        format = SampleBranchFormat()
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register(format)
+        self.assertEquals([format], self.registry._get_all())
+
+    def test_register_extra(self):
+        format = SampleExtraBranchFormat()
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register_extra(format)
+        self.assertEquals([format], self.registry._get_all())
+
+    def test_register_extra_lazy(self):
+        self.assertEquals([], self.registry._get_all())
+        self.registry.register_extra_lazy("bzrlib.tests.test_branch",
+            "SampleExtraBranchFormat")
+        formats = self.registry._get_all()
+        self.assertEquals(1, len(formats))
+        self.assertIsInstance(formats[0], SampleExtraBranchFormat)
+
+
 #Used by TestMetaDirBranchFormatFactory 
 FakeLazyFormat = None
 

